@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2013 The Cacti Group                                 |
+ | Copyright (C) 2004-2017 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -87,6 +87,7 @@
 #include <math.h>
 #include <mysql.h>
 #include <netdb.h>
+#include <semaphore.h>
 #include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -125,23 +126,16 @@
 #  include <priv.h>
 #endif
 
-#ifdef USE_NET_SNMP
- #undef PACKAGE_NAME
- #undef PACKAGE_VERSION
- #undef PACKAGE_BUGREPORT
- #undef PACKAGE_STRING
- #undef PACKAGE_TARNAME
- #include <net-snmp/net-snmp-config.h>
- #include <net-snmp/utilities.h>
- #include <net-snmp/net-snmp-includes.h>
- #include <net-snmp/config_api.h>
- #include <net-snmp/mib_api.h>
-#else
- #include <ucd-snmp/ucd-snmp-config.h>
- #include <ucd-snmp/ucd-snmp-includes.h>
- #include <ucd-snmp/system.h>
- #include <mib.h>
-#endif
+#undef PACKAGE_NAME
+#undef PACKAGE_VERSION
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#include <net-snmp/net-snmp-config.h>
+#include <net-snmp/utilities.h>
+#include <net-snmp/net-snmp-includes.h>
+#include <net-snmp/config_api.h>
+#include <net-snmp/mib_api.h>
 
 #ifdef HAVE_LCAP
 #  include <sys/capability.h>
