@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2013 The Cacti Group                                 |
+ | Copyright (C) 2004-2017 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -51,6 +51,14 @@
  *
  *	Stop polling after device <ID> (else ends with the last one)
  *
+ * -m | --mibs
+ *
+ *	Collect all system mibs this pass
+ *
+ * -N | --mode=online|offline|recovery
+ *
+ *	For remote pollers, the polling mode.  The default is 'online'
+ *
  * -H | --hostlist="hostid1,hostid2,hostid3,...,hostidn"
  *
  *	Override the expected first host, last host behavior with a list of hostids.
@@ -92,16 +100,17 @@
 /* Global Variables */
 int entries = 0;
 int num_hosts = 0;
-int active_threads = 0;
-int active_scripts = 0;
-int thread_ready   = FALSE;
+int pending_threads = 0;
+sem_t active_threads;
+sem_t active_scripts;
 
 config_t set;
 php_t	*php_processes = 0;
-char	 config_paths[CONFIG_PATHS][BUFSIZE];
+char	config_paths[CONFIG_PATHS][BUFSIZE];
+int     *debug_devices;
 
 static char *getarg(char *opt, char ***pargv);
-static void display_help(void);
+static void display_help(int only_version);
 
 #ifdef HAVE_LCAP
 /* This patch is adapted (copied) patch for ntpd from Jarno Huuskonen and
@@ -178,18 +187,18 @@ void drop_root(uid_t server_uid, gid_t server_gid) {
 int main(int argc, char *argv[]) {
 	struct timeval now;
 	char *conf_file = NULL;
-	double begin_time, end_time, current_time;
+	double begin_time, end_time, cur_time;
 	int num_rows = 0;
 	int device_counter = 0;
-	int poller_counter = 0;
-	int last_active_threads = 0;
 	int valid_conf_file = FALSE;
-	long int EXTERNAL_THREAD_SLEEP = 50;
-	long int internal_thread_sleep;
 	char querybuf[BIG_BUFSIZE], *qp = querybuf;
 	char *host_time = NULL;
+	double host_time_double = 0;
 	int itemsPT = 0;
 	int device_threads;
+	sem_t thread_init_sem;
+	int a_threads_value;
+	struct timespec until;
 
 	#ifdef HAVE_LCAP
 	if (geteuid() == 0)
@@ -208,7 +217,6 @@ int main(int argc, char *argv[]) {
 	int canexit = FALSE;
 	int host_id = 0;
 	int i;
-	int mutex_status  = 0;
 	int thread_status = 0;
 	int change_host   = TRUE;
 	int current_thread;
@@ -224,6 +232,9 @@ int main(int argc, char *argv[]) {
 		php_processes[i].php_state = PHP_BUSY;
 	}
 
+	/* create the array of debug devices */
+	debug_devices = calloc(100, sizeof(int));
+
 	/* initialize icmp_avail */
 	set.icmp_avail = TRUE;
 
@@ -278,13 +289,14 @@ int main(int argc, char *argv[]) {
 	 */
 
 	/* initialize some global variables */
-	set.poller_id         = 0;
+	set.poller_id         = 1;
 	set.start_host_id     = -1;
 	set.end_host_id       = -1;
 	set.host_id_list[0]   = '\0';
 	set.php_initialized   = FALSE;
 	set.logfile_processed = FALSE;
 	set.parent_fork       = SPINE_PARENT;
+	set.mode              = REMOTE_ONLINE;
 
 	for (argv++; *argv; argv++) {
 		char	*arg = *argv;
@@ -323,16 +335,44 @@ int main(int argc, char *argv[]) {
 			set.poller_id = atoi(getarg(opt, &argv));
 		}
 
+		else if (STRMATCH(arg, "-N") ||
+				 STRIMATCH(arg, "--mode")) {
+
+			if (STRIMATCH(getarg(opt, &argv), "online")) {
+				set.mode = REMOTE_ONLINE;
+			} 
+
+			else if (STRIMATCH(getarg(opt, &argv), "offline")) {
+				set.mode = REMOTE_OFFLINE;
+			} 
+
+			else if (STRIMATCH(getarg(opt, &argv), "recovery")) {
+				set.mode = REMOTE_RECOVERY;
+			}
+
+			else {
+				die("ERROR: invalid polling mode '%s' specified", opt);
+			}
+		}
+
 		else if (STRMATCH(arg, "-H") ||
 				 STRIMATCH(arg, "--hostlist")) {
 			snprintf(set.host_id_list, BIG_BUFSIZE, "%s", getarg(opt, &argv));
 		}
 
-		else if (STRMATCH(arg, "-h") ||
-				 STRMATCH(arg, "-v") ||
-				 STRMATCH(arg, "--help") ||
-				 STRMATCH(arg, "--version")) {
-			display_help();
+		else if (STRMATCH(arg, "-M") ||
+				 STRMATCH(arg, "--mibs")) {
+			set.mibs = 1;
+		}
+
+		else if (STRMATCH(arg, "-h") || STRMATCH(arg, "--help")) {
+			display_help(FALSE);
+
+			exit(EXIT_SUCCESS);
+		}
+
+		else if (STRMATCH(arg, "-v") || STRMATCH(arg, "--version")) {
+			display_help(TRUE);
 
 			exit(EXIT_SUCCESS);
 		}
@@ -459,12 +499,28 @@ int main(int argc, char *argv[]) {
 		set.poller_interval = 300;
 	}
 
-	/* calculate the external_tread_sleep value */
-	internal_thread_sleep = EXTERNAL_THREAD_SLEEP * set.num_parent_processes / 50;
+	/* tokenize the debug devices */
+	if (strlen(set.selective_device_debug)) {
+		SPINE_LOG_DEBUG(("Selective Debug Devices %s", set.selective_device_debug));
+		int i = 0;
+		char *token = strtok(set.selective_device_debug, ",");
+		while(token) {
+			debug_devices[i]   = atoi(token);
+			debug_devices[i+1] = '\0';
+			token = strtok(NULL, ",");
+			i++;
+		}
+	}else{
+		debug_devices[0] = '\0';
+	}
 
 	/* connect to database */
 	db_connect(set.dbdb, &mysql);
 
+	/* Since MySQL 5.7 the sql_mode defaults are too strict for cacti */
+	db_insert(&mysql, "SET SESSION sql_mode = (SELECT REPLACE(@@sql_mode,'NO_ZERO_DATE', ''))");
+	db_insert(&mysql, "SET SESSION sql_mode = (SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY', ''))");
+
 	if (set.log_level == POLLER_VERBOSITY_DEBUG) {
 		SPINE_LOG_DEBUG(("Version %s starting", VERSION));
 	}else{
@@ -513,6 +569,7 @@ int main(int argc, char *argv[]) {
 			SPINE_LOG(("WARNING: PollerID > 0, but 'host' table does NOT contain the poller_id column!!"));
 		}
 	}
+	db_free_result(result);
 
 	/* determine if the device_threads field exists in the host table */
 	result = db_query(&mysql, "SHOW COLUMNS FROM host LIKE 'device_threads'");
@@ -521,6 +578,7 @@ int main(int argc, char *argv[]) {
 	}else{
 		set.device_threads_exists = FALSE;
 	}
+	db_free_result(result);
 
 	if (set.device_threads_exists) {
 		SPINE_LOG_MEDIUM(("NOTE: Spine will support multithread device polling."));
@@ -546,7 +604,6 @@ int main(int argc, char *argv[]) {
 	qp += sprintf(qp, " ORDER BY id");
 
 	result = db_query(&mysql, querybuf);
-
 	if (set.poller_id == 0) {
 		num_rows = mysql_num_rows(result) + 1; /* add 1 for host = 0 */
 	}else{
@@ -562,13 +619,8 @@ int main(int argc, char *argv[]) {
 	}
 
 	/* mark the spine process as started */
-	if (!set.pre087g) {
-		SPINE_LOG_MEDIUM(("NOTE: Spine is behaving in a 0.8.7g+ manner"));
-		snprintf(querybuf, BIG_BUFSIZE, "INSERT INTO poller_time (poller_id, pid, start_time, end_time) VALUES (%i, %i, NOW(), '0000-00-00 00:00:00')", set.poller_id, getpid());
-		db_insert(&mysql, querybuf);
-	}else{
-		SPINE_LOG_MEDIUM(("NOTE: Spine is behaving in a PRE 0.8.7g manner"));
-	}
+	snprintf(querybuf, BIG_BUFSIZE, "INSERT INTO poller_time (poller_id, pid, start_time, end_time) VALUES (%i, %i, NOW(), '0000-00-00 00:00:00')", set.poller_id, getpid());
+	db_insert(&mysql, querybuf);
 
 	/* initialize threads and mutexes */
 	pthread_attr_init(&attr);
@@ -576,7 +628,21 @@ int main(int argc, char *argv[]) {
 
 	init_mutexes();
 
-	SPINE_LOG_DEBUG(("DEBUG: Initial Value of Active Threads is %i", active_threads));
+	/* initialize active_threads semaphore */
+	sem_init(&active_threads, 0, set.threads);
+
+	/* initialize active_scripts semaphore */
+	sem_init(&active_scripts, 0, MAX_SIMULTANEOUS_SCRIPTS);
+
+	/* initialize thread initialization semaphore */
+	sem_init(&thread_init_sem, 0, 1);
+
+	/* specify the point of timeout for timedwait semaphores */
+	until.tv_sec = (time_t)(set.poller_interval + begin_time - 0.2);
+	until.tv_nsec = 0;
+
+	sem_getvalue(&active_threads, &a_threads_value);
+	SPINE_LOG_MEDIUM(("DEBUG: Initial Value of Active Threads is %i", set.threads - a_threads_value));
 
 	/* tell fork processes that they are now active */
 	set.parent_fork = SPINE_FORK;
@@ -594,178 +660,130 @@ int main(int argc, char *argv[]) {
 	}
 
 	/* loop through devices until done */
-	while ((device_counter < num_rows) && (canexit == FALSE)) {
-		while ((active_threads < set.threads) && (device_counter < num_rows) && (canexit == FALSE)) {
-			mutex_status = thread_mutex_trylock(LOCK_THREAD);
-	
-			switch (mutex_status) {
+	while (canexit == FALSE && device_counter < num_rows) {
+		if (change_host) {
+			mysql_row       = mysql_fetch_row(result);
+			host_id         = atoi(mysql_row[0]);
+			device_threads  = atoi(mysql_row[1]);
+			current_thread  = 1;
+		}else{
+			current_thread++;
+		}
+
+		change_host = (current_thread >= device_threads) ? TRUE : FALSE;
+
+		/* determine how many items will be polled per thread */
+		if (device_threads > 1) {
+			if (current_thread == 1) {
+				snprintf(querybuf, BIG_BUFSIZE, "SELECT CEIL(COUNT(*)/%i) FROM poller_item WHERE host_id=%i AND rrd_next_step <=0", device_threads, host_id);
+				tresult   = db_query(&mysql, querybuf);
+				mysql_row = mysql_fetch_row(tresult);
+
+				itemsPT   = atoi(mysql_row[0]);
+				db_free_result(tresult);
+
+				if (host_time) free(host_time);
+				host_time = get_host_poll_time();
+				host_time_double = get_time_as_double();
+			}
+		}else{
+			itemsPT   = 0;
+			if (host_time) free(host_time);
+			host_time = get_host_poll_time();
+			host_time_double = get_time_as_double();
+		}
+
+		/* populate the thread structure */
+		if (!(poller_details = (poller_thread_t *)malloc(sizeof(poller_thread_t)))) {
+			die("ERROR: Fatal malloc error: spine.c poller_details!");
+		}
+
+		poller_details->host_id          = host_id;
+		poller_details->host_thread      = current_thread;
+		poller_details->last_host_thread = device_threads;
+		poller_details->host_data_ids    = itemsPT;
+		poller_details->host_time        = host_time;
+		poller_details->host_time_double = host_time_double;
+		poller_details->thread_init_sem  = &thread_init_sem;
+
+		retry1:
+
+		if (sem_timedwait(&active_threads, &until) == -1) {
+			if (errno == ETIMEDOUT) {
+				SPINE_LOG(("ERROR: Spine Timed Out While Processing Devices Internal"));
+				canexit = TRUE;
+				break;
+			}else if (errno == EINTR) {
+				usleep(10000);
+				goto retry1;
+			}
+		}
+
+		/* create child process */
+		sem_wait(&thread_init_sem);
+
+		thread_status = pthread_create(&threads[device_counter], &attr, child, poller_details);
+
+		switch (thread_status) {
 			case 0:
-				last_active_threads = active_threads;
-	
+				SPINE_LOG_DEBUG(("DEBUG: Valid Thread to be Created"));
+
 				if (change_host) {
-					mysql_row       = mysql_fetch_row(result);
-					host_id         = atoi(mysql_row[0]);
-					device_threads  = atoi(mysql_row[1]);
-					current_thread  = 1;
-				}else{
-					current_thread++;
+					device_counter++;
 				}
 
-				if (current_thread >= device_threads) {
-					change_host = TRUE;
-				}else{
-					change_host = FALSE;
-				}
+				sem_wait(&thread_init_sem);
+				thread_mutex_lock(LOCK_PEND);
+				pending_threads++;
 
-				/* determine how many items will be polled per thread */
-				if (device_threads > 1) {
-					if (current_thread == 1) {
-						snprintf(querybuf, BIG_BUFSIZE, "SELECT CEIL(COUNT(*)/%i) FROM poller_item WHERE host_id=%i", device_threads, host_id);
-						tresult   = db_query(&mysql, querybuf);
-						mysql_row = mysql_fetch_row(tresult);
-						itemsPT   = atoi(mysql_row[0]);
-						if (host_time) free(host_time);
-						host_time = get_host_poll_time();
-					}
-				}else{
-					itemsPT   = 0;
-					if (host_time) free(host_time);
-					host_time = get_host_poll_time();
-				}
+				sem_getvalue(&active_threads, &a_threads_value);
+				SPINE_LOG_MEDIUM(("SPINE: Active Threads is %i, Pending is %i", set.threads - a_threads_value, pending_threads));
 
-				/* populate the thread structure */
-				if (!(poller_details = (poller_thread_t *)malloc(sizeof(poller_thread_t)))) {
-					die("ERROR: Fatal malloc error: spine.c poller_details!");
-				}
+				thread_mutex_unlock(LOCK_PEND);
+				sem_post(&thread_init_sem);
 
-				poller_details->host_id          = host_id;
-				poller_details->host_thread      = current_thread;
-				poller_details->last_host_thread = device_threads;
-				poller_details->host_data_ids    = itemsPT;
-				poller_details->host_time        = host_time;
-
-				/* this variable tells us that the child had loaded the poller
-				 * poller_details structure and we can move on to the next thread
-				 */
-				thread_ready = FALSE;
-
-				/* create child process */
-				thread_status = pthread_create(&threads[device_counter], &attr, child, poller_details);
-
-				switch (thread_status) {
-					case 0:
-						SPINE_LOG_DEBUG(("DEBUG: Valid Thread to be Created"));
-
-						if (change_host) {
-							device_counter++;
-						}
-						active_threads++;
-
-						/* wait for the child to read and process the structure */
-						while (!thread_ready) { 
-							usleep(internal_thread_sleep);
-						}
-
-						SPINE_LOG_DEBUG(("DEBUG: The Value of Active Threads is %i", active_threads));
-
-						break;
-					case EAGAIN:
-						SPINE_LOG(("ERROR: The System Lacked the Resources to Create a Thread"));
-						break;
-					case EFAULT:
-						SPINE_LOG(("ERROR: The Thread or Attribute were Invalid"));
-						break;
-					case EINVAL:
-						SPINE_LOG(("ERROR: The Thread Attribute is Not Initialized"));
-						break;
-					default:
-						SPINE_LOG(("ERROR: Unknown Thread Creation Error"));
-						break;
-				}
-
-				thread_mutex_unlock(LOCK_THREAD);
-
-				/* get current time and exit program if time limit exceeded */
-				if (poller_counter >= 20) {
-					current_time = get_time_as_double();
-
-					if ((current_time - begin_time + .2) > set.poller_interval) {
-						SPINE_LOG(("ERROR: Spine Timed Out While Processing Hosts Internal"));
-						canexit = TRUE;
-						break;
-					}
-
-					poller_counter = 0;
-				}else{
-					poller_counter++;
-				}
-	
 				break;
-			case EDEADLK:
-				SPINE_LOG(("ERROR: Deadlock Occured"));
-				break;
-			case EBUSY:
-				break;
-			case EINVAL:
-				SPINE_LOG(("ERROR: Attempt to Unlock an Uninitialized Mutex"));
+			case EAGAIN:
+				SPINE_LOG(("ERROR: The System Lacked the Resources to Create a Thread"));
 				break;
 			case EFAULT:
-				SPINE_LOG(("ERROR: Attempt to Unlock an Invalid Mutex"));
+				SPINE_LOG(("ERROR: The Thread or Attribute were Invalid"));
+				break;
+			case EINVAL: SPINE_LOG(("ERROR: The Thread Attribute is Not Initialized"));
 				break;
 			default:
-				SPINE_LOG(("ERROR: Unknown Mutex Lock Error Code Returned"));
+				SPINE_LOG(("ERROR: Unknown Thread Creation Error"));
 				break;
-			}
-	
-			/* get current time and exit program if time limit exceeded */
-			if (poller_counter >= 20) {
-				current_time = get_time_as_double();
-	
-				if ((current_time - begin_time + .2) > set.poller_interval) {
-					SPINE_LOG(("ERROR: Spine Timed Out While Processing Hosts Internal"));
-					canexit = TRUE;
-					break;
-				}
-	
-				poller_counter = 0;
-			}else{
-				poller_counter++;
-			}
 		}
 
-		usleep(internal_thread_sleep);
+		/* Restore thread initialization semaphore if thread creation failed */
+		if (thread_status)
+			sem_post(&thread_init_sem);
 	}
 
-	/* wait for all threads to complete */
-	while (canexit == FALSE) {
-		if (thread_mutex_trylock(LOCK_THREAD) == 0) {
-			last_active_threads = active_threads;
+	/* wait for all threads to 'complete'
+ 	 * using the mutex here as the semaphore will
+     * show zero before the children are done */
+	while (TRUE) {
+		thread_mutex_lock(LOCK_PEND);
 
-			if (active_threads == 0) {
-				canexit = TRUE;
-			}
+		cur_time = get_time_as_double();
 
-			thread_mutex_unlock(LOCK_THREAD);
+		if (pending_threads == 0) {
+			break;
+		} else if (cur_time - begin_time > set.poller_interval) {
+			SPINE_LOG(("ERROR: Spine Timed Out While Waiting for Threads to End"));
+			break;
 		}
 
-		usleep(EXTERNAL_THREAD_SLEEP);
+		thread_mutex_unlock(LOCK_PEND);
 
-		/* get current time and exit program if time limit exceeded */
-		if (poller_counter >= 20) {
-			current_time = get_time_as_double();
-
-			if ((current_time - begin_time + .2) > set.poller_interval) {
-				SPINE_LOG(("ERROR: Spine Timed Out While Processing Hosts Internal"));
-				canexit = TRUE;
-				break;
-			}
-
-			poller_counter = 0;
-		}else{
-			poller_counter++;
-		}
+		sleep(1);
 	}
 
+	sem_getvalue(&active_threads, &a_threads_value);
+	SPINE_LOG_MEDIUM(("SPINE: The Final Value of Threads is %i", set.threads - a_threads_value));
+
 	/* tell Spine that it is now parent */
 	set.parent_fork = SPINE_PARENT;
 
@@ -773,14 +791,11 @@ int main(int argc, char *argv[]) {
 	gettimeofday(&now, NULL);
 
 	/* update the db for |data_time| on graphs */
-	db_insert(&mysql, "replace into settings (name,value) values ('date',NOW())");
-
-	/* setup poller_time depending on Cacti version */
-	if (set.pre087g) {
-		snprintf(querybuf, BIG_BUFSIZE, "INSERT INTO poller_time (poller_id, pid, start_time, end_time) VALUES (%i, %i, NOW(), NOW())", set.poller_id, getpid());
-	}else{
-		snprintf(querybuf, BIG_BUFSIZE, "UPDATE poller_time SET end_time=NOW() WHERE poller_id=%i AND pid=%i", set.poller_id, getpid());
+	if (set.poller_id == 1) {
+		db_insert(&mysql, "REPLACE INTO settings (name,value) VALUES ('date',NOW())");
 	}
+
+	snprintf(querybuf, BIG_BUFSIZE, "UPDATE poller_time SET end_time=NOW() WHERE poller_id=%i AND pid=%i", set.poller_id, getpid());
 	db_insert(&mysql, querybuf);
 
 	/* cleanup and exit program */
@@ -799,11 +814,12 @@ int main(int argc, char *argv[]) {
 	free(threads);
 	free(ids);
 	free(conf_file);
+	free(debug_devices);
 
 	SPINE_LOG_DEBUG(("DEBUG: Allocated Variable Memory Freed"));
 
 	/* close mysql */
-	mysql_free_result(result);
+	db_free_result(result);
 	mysql_close(&mysql);
 
 	SPINE_LOG_DEBUG(("DEBUG: MYSQL Free & Close Completed"));
@@ -817,11 +833,11 @@ int main(int argc, char *argv[]) {
 	end_time = TIMEVAL_TO_DOUBLE(now);
 
 	if (set.log_level >= POLLER_VERBOSITY_MEDIUM) {
-		SPINE_LOG(("Time: %.4f s, Threads: %i, Hosts: %i", (end_time - begin_time), set.threads, num_rows));
+		SPINE_LOG(("Time: %.4f s, Threads: %i, Devices: %i", (end_time - begin_time), set.threads, num_rows));
 	}else{
 		/* provide output if running from command line */
 		if (!set.stdout_notty) {
-			fprintf(stdout,"SPINE: Time: %.4f s, Threads: %i, Hosts: %i\n", (end_time - begin_time), set.threads, num_rows);
+			fprintf(stdout,"SPINE: Time: %.4f s, Threads: %i, Devices: %i\n", (end_time - begin_time), set.threads, num_rows);
 		}
 	}
 
@@ -839,7 +855,7 @@ int main(int argc, char *argv[]) {
  *	is dumped literally.
  *
  */
-static void display_help(void) {
+static void display_help(int only_version) {
 	static const char *const *p;
 	static const char * const helptext[] = {
 		"Usage: spine [options] [[firstid lastid] || [-H/--hostlist='hostid1,hostid2,...,hostidn']]",
@@ -852,6 +868,10 @@ static void display_help(void) {
 		"  -p/--poller=X      Set the poller id to X",
 		"  -C/--conf=F        Read spine configuration from file F",
 		"  -O/--option=S:V    Override DB settings 'set' with value 'V'",
+		"  -M/--mibs          Refresh the device System Mib data",
+		"  -N/--mode=online   For remote pollers, the operating mode.",
+		"                     Options include: online, offline, recovery.",
+		"                     The default is 'online'.",
 		"  -R/--readonly      Spine will not write output to the DB",
 		"  -S/--stdout        Logging is performed to standard output",
 		"  -V/--verbosity=V   Set logging verbosity to <V>",
@@ -876,10 +896,13 @@ static void display_help(void) {
 		0 /* ENDMARKER */
 	};
 
-	printf("SPINE %s  Copyright 2002-2013 by The Cacti Group\n\n", VERSION);
+	printf("SPINE %s  Copyright 2004-2017 by The Cacti Group\n", VERSION);
 
-	for (p = helptext; *p; p++) {
-		puts(*p);	/* automatically adds a newline */
+	if (only_version == FALSE) {
+		printf("\n");
+		for (p = helptext; *p; p++) {
+			puts(*p);	/* automatically adds a newline */
+		}
 	}
 }
 
